/*
 * Copyright 2004-2005 the Seasar Foundation and the Others.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, 
 * either express or implied. See the License for the specific language
 * governing permissions and limitations under the License.
 */
package org.seasar.maya.sample.hangman;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.Random;

/**
 * original code from Apache Jakarta Tapestry tutorial.
 */
public class HangmanGame implements CONST_GAME, Serializable {
	
    private static final long serialVersionUID = -4507735058166778682L;
    private static Random r = new Random();

    private int _maxMisses = 5;
	private int _missed = 0;
	private String _currentWord = "";
	private ArrayList _allUnusedLetters = null;
	private String _guess = "";
	private String _guessedRightLetters = "";
	private boolean _gameComplete = false;
    private String _lastError = "";

	public void start(int maxMisses) {
		_maxMisses = maxMisses;
		_missed = 0;
		_gameComplete = false;
		_currentWord = LIST_WORD[r.nextInt(LIST_WORD.length)].toUpperCase();
		_guessedRightLetters = "";
		_guess = "";
		for (int count = 0; count < _currentWord.length(); count++) {
			_guess += "_ ";
		}
		_allUnusedLetters = new ArrayList();
		String sLetters = "ABCDEFGHIJKLMNOPQRSTUVWXYZ";
		for (int count = 0; count < sLetters.length(); count++) {
			_allUnusedLetters.add(sLetters.substring(count, count + 1));
		}
        _lastError = "";
	}

	public String getCorrectWord() {
		return _currentWord;
	}

	public boolean getGameComplete() {
		return _gameComplete;
	}
	public void setGameComplete(boolean value) {
		_gameComplete = value;
	}

	//The current state of the guessed word
	public String getCurrent() {
		return _guess;
	}

	public int getMaxMisses() {
		return _maxMisses;
	}

	public int getMissed() {
		return _missed;
	}

	public ArrayList getUnused() {
		return _allUnusedLetters;
	}
    
    public String getImageName() {
        int missed = getMissed();
        int maxMisses = getMaxMisses();
        int factor = 1;
        if(maxMisses==3) {
            factor = 3;    
        } else if(maxMisses==5) {
            factor = 2;    
        }
        return PREFIX_IMAGE + (missed * factor) + SUFFIX_IMAGE;           
    }

    public void setLastError(String lastError) {
        _lastError = lastError;
    }
    
    public String getLastError() {
        return _lastError;
    }
    
	public boolean guess(String letter) {
		//Remove the letter from the "unused letter" list
		for (int count = 0; count < _allUnusedLetters.size(); count++) {
			if (((String) _allUnusedLetters.get(count)).equals(letter)) {
				_allUnusedLetters.remove(count);
				break;
			}
		}
		//Is the letter in the word?
		if (_currentWord.indexOf(letter) == -1) {
			//This letter is NOT in the word
			_missed++;
			return false;
		}
		//Good letter! Reveal it in the guess
		_guessedRightLetters += letter;

		//Rebuild the guessed word... make sure
		//guessed letters are visible.
		_guess = "";
		for (int count = 0; count < _currentWord.length(); count++) {
			String find = _currentWord.substring(count, count + 1);
			if (_guessedRightLetters.indexOf(find) == -1) {
				_guess += "_ ";
			} else {
				_guess += find + " ";
			}
		}

        return true;
	}

	public boolean getFailed() {
		if (_missed > _maxMisses) {
			return (true);
		}
		return (false);
	}

	public boolean getDone() {
		if (_guess.indexOf("_") == -1) {
			//All letters guessed right!
			return (true);
		}
		return (false);
	}

}